export class ContentEditHistoryManager {
  constructor(options) {
    this.pageIndex = options.pageIndex
    this.messageHandler = options.messageHandler
    this.eventBus = options.eventBus
    this.pdfViewer = options.pdfViewer

    this.undoList = []
    this.redoList = []
    this.editPagePtr = 0
  }

  async canRedo() {
    if (!this.editPagePtr) {
      console.warn('Not in content edit mode')
      return false
    }

    const res = await this.messageHandler.sendWithPromise('CanRedo', this.editPagePtr)
    return !!res
  }

  async canUndo() {
    if (!this.editPagePtr) {
      console.warn('Not in content edit mode')
      return false
    }

    const res = await this.messageHandler.sendWithPromise('CanUndo', this.editPagePtr)
    return !!res
  }

  async redo() {
    if (!this.editPagePtr) {
      console.warn('Not in content edit mode')
      return false
    }

    const lastRedo = this.redoList.pop()
    if (!lastRedo) {
      console.warn('No actions to redo')
      return false
    }
    const res = await this.messageHandler.sendWithPromise('Redo', this.editPagePtr)

    this.undoList.push(lastRedo)
    this.updateView(res)

    this.eventBus.dispatch('undoRedoStatusChanged')
  }

  async undo() {
    if (!this.editPagePtr) {
      console.warn('Not in content edit mode')
      return false
    }

    const lastUndo = this.undoList.pop()
    if (!lastUndo) {
      console.warn('No actions to undo')
      return false
    }
    const res = await this.messageHandler.sendWithPromise('Undo', this.editPagePtr)
    const canRedo = await this.canRedo()

    canRedo && this.redoList.push(lastUndo)
    this.updateView(res)

    this.eventBus.dispatch('undoRedoStatusChanged')
  }

  async updateView(res) {
    await this.pdfViewer._pages[this.pageIndex].contentContainer.updateFrameEditor(res)

    this.eventBus.dispatch('changeOperateList', {
      undoListLength: this.undoList.length,
      redoListLength: this.redoList.length
    })
  }
}
